<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification; 

class Fai32000DespesasDiversasController extends \Teknisa\Libs\Controller\CRUD{
    protected $dataSourceName = "proddespprev";
    protected $dataSourceManager;
    protected $connection;
    protected $newCode;
    protected $environment;

    
    public function __construct($crudService, $environment, $dataSourceManager, $connection, $newCode) {
        $this->dataSourceManager    = $dataSourceManager;
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        parent::__construct($crudService, $environment);
        
    }
    
    public function excluiDespesa(DTO\Request $request, DTO\Response $response ){
        $dados = $request->getParameter("DataSet");
        
        if($dados != null){
            $despesa = $dados[0];
            
            try{
                $this->connection->beginTransaction();
                $filtro = array(
                    array(
                        "columnName" => "CDDESPPREV",
                        "operator" => "=",
                        "value" => $despesa["CDDESPPREV"]
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $despesa["NRORG"]
                    )
                );
                $dataSetProd = $this->crudService->getFilteredDataSet('proddespprev', $filtro);
    		    $this->dataSourceManager->delete($dataSetProd);
    		    $dataSet = $this->crudService->getFilteredDataSet('despprev', $filtro);
                $this->dataSourceManager->delete($dataSet);
                
                $this->connection->commit();
                
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-02292") === FALSE){
                    $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
                }else{
                    $response->setError(new Error("DESPREGISTROVINCULADO", 500));
                }
                
                $this->connection->rollback();
            }

        }
    }

    public function saveProduto(DTO\Request $request, DTO\Response $response ){

        $rows = $request->getDataSet()->getRows();
        $erroList = array();
        $saved = false;
        if($rows != null){
            try{
                $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
                foreach($rowsToSave as $row){
                    try{
                        $validProd = $this->validaProduto($row["CDPRODUTO"]);
                        if(empty($validProd)){
                            $this->crudService->save($this->dataSourceName, [$row]);
                            $saved = true;
                        }
                        else{
                            foreach($validProd as $prod){
                                array_push($erroList, $prod);
                            }
                        }
                    }catch(\Exception $ex){
                        if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                            array_push($erroList, "FALHA_PROC_DADOS " . $ex->getMessage());
                            //$this->connection->rollback();
                        }else{
                            array_push($erroList, $row["NMPRODUTO"]);
                        }    
                    }
                }
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                    array_push($erroList, "FALHA_PROC_DADOS " . $ex->getMessage());
                    //$this->connection->rollback();
                }else{
                    array_push($erroList, $row["NMPRODUTO"]);
                }
            }
            
            if(!empty($erroList)){
                $response->addDataSet(new DataSet("errorList", $erroList));
                $response->addDataSet(new DataSet("saved", [$saved]));
                $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
            }
        } 
    }
    
    public function saveDespesa(DTO\Request $request, DTO\Response $response ){
        $dataSourceName = "despprev";
        $row = $request->getRow();
        $qtde_prod_insert = 0;
        $qtde_prod = 0;
        $erroList = array();
        $tipo = "";
        if($row != null){
            try{
                $row["IDDESTACAROP"] = "";
                
                if($row["__is_new"]){
                    if(count($row["CDPRODUTO"]) > 0){
                        $qtde_prod = count($row["CDPRODUTO"]);
                        $prod = array();
                        foreach($row["CDPRODUTO"] as $produto){
                            $produtoInsert["NRPRODDESPPREV"] = $this->newCode->getCodeByTableName("PRODDESPPREV", $this->environment->getNrOrg());
                            $produtoInsert["CDPRODUTO"] = $produto;
                            $produtoInsert["CDDESPPREV"] = $row["CDDESPPREV"];
                            $produtoInsert["__is_new"] = true;
                            
                            $valid = $this->validaProduto($produto);
                            if(empty($valid)){
                                array_push($prod, $produtoInsert);
                            }else{
                                foreach($valid as $prodErr){
                                    array_push($erroList, $prodErr);
                                }
                            }
                        }
                        if(!empty($prod)){
                            $dataSet = new DataSet($dataSourceName, [$row]);
                            
                            $request = new DTO\Request\DataSet($dataSet, "", "", "");
                            
                            $rowToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                     
                            $this->crudService->save($dataSourceName, $rowToSave);
                            
                            $dataSetProd = new DataSet($this->dataSourceName, $prod);
                                
                            $requestProd = new DTO\Request\DataSet($dataSetProd, "", "", "");
                            
                            $rowToSaveProduto = $this->crudService->getRowsToSave($requestProd, $this->dataSourceName);
                            
                            $this->crudService->save($this->dataSourceName, $rowToSaveProduto);
                            
                            $qtde_prod_insert = count($prod);
                            $response->addDataSet(new DataSet("PRODINSERT", [$qtde_prod == $qtde_prod_insert]));
                            $response->addDataSet(new DataSet("primaryKey", array("CDDESPPREV"=>$row["CDDESPPREV"])));
                            
                        }else{
                            array_push($erroList, "INFO_UM_PROD_VALID");
                        }
                    }
                }else{
                    if($row['IDTIPODESPESA'] == 'ENTRADA'){
                        $tipo = 'LANCENTRADA';
                    }else{
                        $tipo = $row['IDTIPODESPESA'];
                    }
               
                    $parametros =  array(
                            'P_CDDESPPREV' => $row['CDDESPPREV'],
                            'P_IDTIPODESPESA' => $tipo
                        );
                    $stmt = $this->connection->prepare("SELECT COUNT(*) RESULTADO
                                                          FROM (SELECT CDDESPPREV FROM PERCITEMDETALGOP 
                                                                 WHERE CDDESPPREV = :P_CDDESPPREV
                                                                   AND IDDESPPREV <> :P_IDTIPODESPESA
                                                               UNION
                                                                SELECT CDDESPPREV FROM PERCITDETALFILGOP 
                                                                 WHERE CDDESPPREV = :P_CDDESPPREV
                                                                   AND IDDESPPREV <> :P_IDTIPODESPESA)
                                                            ");
                    $stmt->execute($parametros);
                    $retorno = $stmt->fetchAll();
            	    if($row["FLAG"]){
            	        $dataSet = new DataSet($dataSourceName, [$row]);
                        $request = new DTO\Request\DataSet($dataSet, "", "", "");
                        $rowToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                        $this->crudService->save($dataSourceName, $rowToSave);
            	    }
                    $response->addDataSet(new DataSet("data", $retorno));
                }
            }catch(\Exception $ex){
                
                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                    array_push($erroList, "FALHA_PROC_DADOS " . $ex->getMessage());
                    //$this->connection->rollback();
                }else{
                    array_push($erroList, "DESPJACAD");
                }
            }
            
            if(!empty($erroList) && !(($qtde_prod - $qtde_prod_insert) == 0)){
                $response->addDataSet(new DataSet("errorList", $erroList));
                 $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
                //throw new \Exception ("Ocorreram erros durante a operação. Verifique o log");
            }
        } 
    }
    
    function maskProd($mask, $cod){
        
        $mask[0] = '*';
        $mask = str_replace("9", "*", $mask);
    
        for($i=0;$i<strlen($cod);$i++){
            $mask[strpos($mask,"*")] = $cod[$i];
        }
        if(strpos($mask, '*')){
            $mask= substr_replace($mask, '', strpos($mask, '.*'));
        }
        return $mask;
    }
    
    public function validaProduto($cdProduto){
        $duplicatedProd = array();
        //substr($cdProduto, 0, 3)
        $parametros =  array(
                            'P_CDPRODUTO' => $cdProduto
                        );
        $stmt = $this->connection->prepare("
          SELECT P.CDARVPROD, P.NMPRODUTO, PR.CDPRODUTO, PR.CDDESPPREV, D.NMDESPPREV 
            FROM PRODUTO P, PRODDESPPREV PR, DESPPREV D
           WHERE P.CDPRODUTO  = PR.CDPRODUTO
             AND P.CDARVPROD LIKE (SELECT P.CDARVPROD
                                     FROM PRODUTO P
                                    WHERE P.CDPRODUTO = :P_CDPRODUTO)||'%'
             AND PR.CDDESPPREV = D.CDDESPPREV"); 
        $stmt->execute($parametros);
        
        $result = $stmt->fetchAll();
        
        if(!empty($result)){
            
            try{
                $params = array('NRORG' => $this->environment->getNrorg());
                $stmtMask = $this->connection->prepare("SELECT CDPICTPROD MASK FROM PARAMGERAL WHERE NRORG = :NRORG"); 
                $stmtMask->execute($params);
    
                $mask = $stmtMask->fetchAll()[0]['MASK'];
                
                foreach($result as $produto){
                    $produto['CDARVPROD'] = $this->maskProd($mask, $produto['CDARVPROD']);
                    
                    //array_push($duplicatedProd, "Produto ".$produto['CDARVPROD'] . " - ". $produto['NMPRODUTO']." não pôde ser incluído, pois já está cadastrado na despesa ".$produto['CDDESPPREV'].".");
                    array_push($duplicatedProd, array('CDARVPROD' => $produto['CDARVPROD'],
                                                      'NMPRODUTO' => $produto['NMPRODUTO'],
                                                      'CDDESPPREV' => $produto['CDDESPPREV']
                                                      ));
                }
            }catch(\Exception $ex){
                $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
            }
        }
        return $duplicatedProd;
    }
    
    public function verificaItem(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();
        $resp = 0;
        foreach($rows as $row){
            $parametros =  array(
                            'P_CDDESPPREV' => $rows[0]['CDDESPPREV']
                        );
            $stmt = $this->connection->prepare("SELECT MAX(RESULTADO.GERAL) EXISTE 
                                                    FROM (SELECT COUNT(1) AS GERAL 
                                                            FROM PERCITEMDETALGOP IG 
                                                           WHERE IG.CDDESPPREV = :P_CDDESPPREV
                                               UNION ALL
                                                SELECT COUNT(1) AS GERAL 
                                                    FROM PERCITDETALFILGOP I 
                                                 WHERE I.CDDESPPREV = :P_CDDESPPREV) 
                                                                            RESULTADO"); 
            $stmt->execute($parametros);
            $retorno = $stmt->fetchAll(); 
    	    $return[1] = $retorno[0]["EXISTE"];
    	    if($return[1]>0){
    		    $resp = 1;        
    	    }
        }
        if($resp != 0){
            $response->addDataSet(new DataSet("RESULTADO", $return)); 
        }else{
            $return[1] = 0;
            $response->addDataSet(new DataSet("RESULTADO", $return));
        }
    }
    
    public function excluirItemPercentual(DTO\Request $request, DTO\Response $response ){
       $rows = $request->getDataSet()->getRows();
       if(!empty($rows)){
           foreach ($rows as $row) {
               $parametros =  array(
                            'P_CDDESPPREV' => $row['CDDESPPREV'],
                            'P_CDPRODUTO'  => $row['CDPRODUTO']
                        );
           
               $stmt = $this->connection->prepare("SELECT PG.NRPERCITEMDETALGOP, PG.CDPRODUTO
                                                   FROM PERCDETALPRODGOP PG,PERCITEMDETALGOP IG 
                                                 WHERE IG.CDDESPPREV = :P_CDDESPPREV
                                                   AND PG.CDPRODUTO = :P_CDPRODUTO  
                                                   AND IG.NRPERCITEMDETALGOP = PG.NRPERCITEMDETALGOP
                                              ");
                                              
                $stmtUnid = $this->connection->prepare("SELECT PU.NRPERCITDETALFILGOP, PU.CDPRODUTO  
                                                       FROM PERCDETPRODFILGOP PU,PERCITDETALFILGOP IU 
                                                    WHERE IU.CDDESPPREV = :P_CDDESPPREV 
                                                      AND PU.CDPRODUTO = :P_CDPRODUTO
                                                      AND IU.NRPERCITDETALFILGOP = PU.NRPERCITDETALFILGOP
                                              ");
                $stmt->execute($parametros);
                $retorno = $stmt->fetchAll();
                $stmtUnid->execute($parametros);
                $retornoUnid = $stmtUnid->fetchAll();
                
                if(!empty($retorno[0]["NRPERCITEMDETALGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITEMDETALGOP",
                            "operator" => "=",
                            "value" => $retorno[0]["NRPERCITEMDETALGOP"]
                        ),
                        array(
                            "columnName" => "CDPRODUTO",
                            "operator" => "=",
                            "value" => $retorno[0]["CDPRODUTO"]
                        )
                    );
                    $dataSetProd = $this->crudService->getFilteredDataSet('percdetalprodgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
                if(!empty($retornoUnid[0]["NRPERCITDETALFILGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITDETALFILGOP",
                            "operator" => "=",
                            "value" => $retornoUnid[0]["NRPERCITDETALFILGOP"]
                        ),
                        array(
                            "columnName" => "CDPRODUTO",
                            "operator" => "=",
                            "value" => $retorno[0]["CDPRODUTO"]
                        )
                    );
                    $dataSetProd = $this->crudService->getFilteredDataSet('percdetprodfilgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
           }
       }
        
    }


    
}

