<?php
namespace Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;

class Ger80200FaturamentoController extends \Teknisa\Libs\Controller\CRUD {
    
    protected $environment;
    protected $entityManager;
    protected $connection;
    protected $crudService;
    
    public function __construct($crudService, $environment, $entityManager, $connection) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->connection = $connection;
        parent::__construct($crudService, $environment);
        
    }
    
    public function excluirImposto(DTO\Request $request, DTO\Response $response ){
        $dados = $request->getParameter("DataSet");
        
        if($dados != null){
            $imposto = $dados[0];
            $filtro = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $imposto["CDFILIAL"]
                ),
                array(
                    "columnName" => "IDTPIMPOSTOROP",
                    "operator" => "=",
                    "value" => $imposto["IDTPIMPOSTOROP"]
                ),
                array(
                    "columnName" => "CDIMPOSTO",
                    "operator" => "=",
                    "value" => $imposto["CDIMPOSTO"]
                )
            );
		    $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceName, $filtro);
		  
            $this->crudService->delete($response, $dataSet);
  
        }
    }
    
    public function atualizaImposto(DTO\Request $request, DTO\Response $response){
        $dados = $request->getParameter("DataSet");
        
        if($dados != null){
            $impostoBackupRow = $dados[0];
            $impostoCurrentRow  = $dados[1];
            $param = array(
                        'P_IDTPIMPOSTOROP_NOVO' => $impostoCurrentRow["IDTPIMPOSTOROP"],
                        'P_IDTRATIMPROP_NOVO' => $impostoCurrentRow["IDTRATIMPROP"],
                        'P_VRPERFIXCALCIMP_NOVO' => $impostoCurrentRow["VRPERFIXCALCIMP"],
                        'P_CDFILIAL' => $impostoBackupRow["CDFILIAL"],
                        'P_CDIMPOSTO' => $impostoBackupRow["CDIMPOSTO"],
                        'P_IDTPIMPOSTOROP' => $impostoBackupRow["IDTPIMPOSTOROP"],
                        'P_IDTRATIMPROP' => $impostoBackupRow["IDTRATIMPROP"]
                    );
            try{
                $stmt = $this->connection->prepare("
                UPDATE IMPOSTOROP
                    SET
                        IDTPIMPOSTOROP = :P_IDTPIMPOSTOROP_NOVO,
                        IDTRATIMPROP = :P_IDTRATIMPROP_NOVO,
                        VRPERFIXCALCIMP = :P_VRPERFIXCALCIMP_NOVO
                        
                    WHERE
                        CDFILIAL   = :P_CDFILIAL   AND
                        CDIMPOSTO  = :P_CDIMPOSTO  AND
                        IDTPIMPOSTOROP = :P_IDTPIMPOSTOROP AND
                        IDTRATIMPROP = :P_IDTRATIMPROP");
    
                $stmt->execute($param);
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                    $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
                }else{
                    $response->setError(new Error("IMPOSTOTIPO_JA_CAD", 500));
                }
            }
        }
    }
   
    public function salvaServico(DTO\Request $request, DTO\Response $response){
        $row = $request->getParameter("DataSet");
        $erroList = array();
        $count = -1;
        
        if(!empty($row)){
            // var_dump($row['CDSERVFILI']).die();
            foreach($row['CDSERVFILI'] as $cdservfili){
                $count ++;
                $parametros =  array(
                    'P_CDFILIAL' => $row['CDFILIAL'],
                    'P_CDSERVFILI' => $cdservfili,
                    'P_IDTPSERVICOROP' => $row['IDTPSERVICOROP'],
                    'P_NRORG' => $this->environment->getNrorg(),
                    'P_DTINCLUSAO' => $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s"),
                    'P_CDOPERINCLUSAO' => $this->environment->getCdOperador(),
                    'P_NRORGINCLUSAO' => $this->environment->getNrorg()
                );
                $stmt = $this->connection->prepare("
                     INSERT INTO
                        SERVICOROP 
                        (
                            CDFILIAL,
                            CDSERVFILI,
                            IDTPSERVICOROP,
                            NRORG,
                            DTINCLUSAO,
                            CDOPERINCLUSAO,
                            NRORGINCLUSAO
                         ) 
                    VALUES(
                        :P_CDFILIAL,
                        :P_CDSERVFILI,
                        :P_IDTPSERVICOROP,
                        :P_NRORG,
                        :P_DTINCLUSAO,
                        :P_CDOPERINCLUSAO,
                        :P_NRORGINCLUSAO
                    )"); 
                try{
                    $stmt->execute($parametros);
                }catch(\Exception $ex){
                    array_push($erroList, ($cdservfili . ' - ' . $row['NMSERVICO'][$count]));
                }
            }
            
            if(!empty($erroList)){
                if(count($erroList) == 1){
                    array_unshift($erroList, 'O serviço a seguir já está cadastrado: ');
                }else{
                    array_unshift($erroList, 'Os serviços a seguir já estão cadastrados: ');
                }
                $response->addDataSet(new DataSet("erroList", $erroList));
            }
            
        }
    }

}


